
# q-expansion of newform 378.3.b.c, downloaded from the LMFDB on 26 April 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 378
weight = 3
poly_data = [16, 0, 0, 0, 1, 0, 0, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0, 0, 0, 0, 0], 1], [[0, 0, 5, 0, 0, 0, 1, 0], 4], [[1, 0, 0, 0, 2, 0, 0, 0], 1], [[0, 0, 3, 0, 0, 0, -1, 0], 4], [[0, 4, 0, -7, 0, -4, 0, 1], 24], [[0, -4, 0, -7, 0, 4, 0, 1], 8], [[0, 44, 0, 13, 0, 4, 0, 5], 24], [[0, 44, 0, -13, 0, 4, 0, -5], 8]]

hecke_ring_character_values = [[29, [-1, 0, 0, 0, 0, 0, 0, 0]], [325, [1, 0, 0, 0, 0, 0, 0, 0]]]
aps_data = [[0, 0, 0, 0, -1, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, 0], [0, -1, 0, 0, 1, 0, -1, 0], [0, 0, 0, -1, 0, 0, 0, 0], [0, -1, 1, 0, -1, 0, 1, 0], [-5, 0, 0, -1, 0, 1, 0, -1], [0, 1, 2, 0, -1, 0, 1, 0], [5, 0, 0, 3, 0, 1, 0, -1], [0, 4, 2, 0, -4, 0, -2, 0], [0, 3, 1, 0, 7, 0, 5, 0], [-5, 0, 0, -1, 0, -3, 0, -3], [-1, 0, 0, 6, 0, 2, 0, -2], [0, 1, 0, 0, 9, 0, -9, 0], [5, 0, 0, 0, 0, -11, 0, -1], [0, -13, -4, 0, 18, 0, 6, 0], [0, -6, 4, 0, -18, 0, -6, 0], [0, -9, 0, 0, -2, 0, -10, 0], [-11, 0, 0, -13, 0, 13, 0, -1], [30, 0, 0, 10, 0, 8, 0, 4], [0, 8, -4, 0, -4, 0, -8, 0], [-36, 0, 0, -14, 0, -4, 0, -2], [-53, 0, 0, -10, 0, 11, 0, 1], [0, 7, 2, 0, 2, 0, 22, 0], [0, -14, 6, 0, 30, 0, -6, 0], [75, 0, 0, -23, 0, -11, 0, 5], [0, -14, 6, 0, -32, 0, 20, 0], [-40, 0, 0, 6, 0, 18, 0, 0], [0, 20, -2, 0, 24, 0, -18, 0], [-27, 0, 0, -14, 0, 30, 0, 6], [0, 15, 11, 0, -5, 0, -19, 0], [-17, 0, 0, -70, 0, 9, 0, 3], [0, -18, 4, 0, -104, 0, -4, 0], [0, -17, 3, 0, 25, 0, 17, 0], [-93, 0, 0, 13, 0, -23, 0, 5], [0, 1, -1, 0, 63, 0, -21, 0], [81, 0, 0, 10, 0, 45, 0, 3], [-67, 0, 0, 51, 0, -21, 0, 3], [89, 0, 0, -12, 0, -5, 0, 5], [0, -1, -14, 0, -38, 0, -22, 0], [0, -26, 16, 0, -32, 0, -4, 0], [0, 1, -19, 0, -51, 0, 9, 0], [-62, 0, 0, -34, 0, -48, 0, -6], [0, 14, -28, 0, -36, 0, 24, 0], [-85, 0, 0, 42, 0, 56, 0, -8], [0, 45, 3, 0, -19, 0, 7, 0], [26, 0, 0, -66, 0, -4, 0, -2], [98, 0, 0, 36, 0, -8, 0, -16], [197, 0, 0, 21, 0, -29, 0, -13], [0, -6, -22, 0, -42, 0, -54, 0], [174, 0, 0, 46, 0, 16, 0, 2], [0, -25, 7, 0, -55, 0, 13, 0], [0, 37, -27, 0, -111, 0, 21, 0], [-19, 0, 0, -51, 0, -29, 0, -19], [0, -55, -22, 0, 30, 0, 6, 0], [0, -28, 0, 0, -186, 0, 30, 0], [0, 21, -19, 0, -33, 0, -27, 0], [0, 61, 2, 0, -17, 0, 5, 0], [-50, 0, 0, -4, 0, 38, 0, -26], [113, 0, 0, -18, 0, 66, 0, -6], [0, 52, -10, 0, 86, 0, -80, 0], [51, 0, 0, 13, 0, -35, 0, 5], [0, -35, 18, 0, -9, 0, 45, 0], [-168, 0, 0, -2, 0, -38, 0, 8], [0, 51, -18, 0, 22, 0, 74, 0], [-54, 0, 0, -98, 0, 12, 0, 6], [0, -24, 30, 0, -114, 0, 12, 0], [-127, 0, 0, 102, 0, -3, 0, 3], [121, 0, 0, -10, 0, -12, 0, 12], [0, 103, 19, 0, 165, 0, 63, 0], [360, 0, 0, -44, 0, -28, 0, 16], [0, 47, 28, 0, 3, 0, -63, 0], [0, -90, -16, 0, -164, 0, 20, 0], [86, 0, 0, 0, 0, -38, 0, 26], [-15, 0, 0, 82, 0, -4, 0, 4], [-69, 0, 0, 106, 0, -73, 0, 13], [0, 15, -26, 0, -292, 0, 16, 0], [0, 25, 15, 0, -101, 0, -109, 0], [148, 0, 0, -22, 0, 24, 0, 30], [0, -114, 10, 0, 144, 0, 6, 0], [-220, 0, 0, -48, 0, -8, 0, 8], [0, -149, -6, 0, -66, 0, 30, 0], [-164, 0, 0, -184, 0, -14, 0, -10], [0, 92, -36, 0, -108, 0, -24, 0], [-307, 0, 0, 69, 0, 73, 0, 5], [-62, 0, 0, 170, 0, 16, 0, 14], [0, -37, 29, 0, -151, 0, -71, 0], [0, 67, 7, 0, 181, 0, 101, 0], [-192, 0, 0, -128, 0, 28, 0, 32], [0, 1, -10, 0, -65, 0, 41, 0], [63, 0, 0, -56, 0, -15, 0, 15], [0, 62, 34, 0, 238, 0, 14, 0], [0, 105, -22, 0, 8, 0, 28, 0], [-220, 0, 0, 84, 0, -48, 0, 36], [0, 67, 3, 0, -135, 0, -117, 0], [165, 0, 0, -170, 0, 9, 0, 15], [0, -23, 4, 0, 144, 0, 108, 0], [0, 5, 6, 0, -627, 0, 3, 0], [0, 145, -22, 0, -189, 0, 21, 0], [-115, 0, 0, -135, 0, 57, 0, -9], [-41, 0, 0, -40, 0, 58, 0, -58], [-59, 0, 0, 128, 0, 47, 0, 13], [0, -120, -8, 0, 64, 0, 32, 0], [0, -10, 38, 0, 392, 0, 4, 0], [0, -120, -6, 0, 6, 0, 72, 0], [355, 0, 0, -100, 0, -31, 0, -17], [210, 0, 0, -2, 0, 32, 0, -14], [0, -96, 52, 0, 2, 0, 58, 0], [0, -145, 46, 0, 345, 0, -141, 0], [0, 79, -35, 0, 375, 0, -111, 0], [257, 0, 0, 45, 0, -21, 0, -39], [-269, 0, 0, -49, 0, -81, 0, -15], [-128, 0, 0, 8, 0, -156, 0, 0], [0, 68, 34, 0, -550, 0, -56, 0], [-321, 0, 0, 229, 0, 71, 0, -11], [-187, 0, 0, 36, 0, 19, 0, -43], [0, -123, -49, 0, -73, 0, 55, 0], [98, 0, 0, -72, 0, 54, 0, -6], [0, -92, -16, 0, -294, 0, -126, 0], [0, 51, -29, 0, 423, 0, -39, 0], [0, 58, 74, 0, -112, 0, 46, 0], [-123, 0, 0, 73, 0, 49, 0, -55], [338, 0, 0, -198, 0, 98, 0, -2], [0, 36, 48, 0, 66, 0, 246, 0], [0, 45, 49, 0, -129, 0, -147, 0], [53, 0, 0, 21, 0, -27, 0, -69], [0, 315, 3, 0, 187, 0, 41, 0], [-97, 0, 0, 240, 0, -122, 0, -10], [0, 43, -36, 0, 162, 0, 150, 0], [231, 0, 0, 133, 0, 91, 0, 41], [-338, 0, 0, 26, 0, -80, 0, -10], [-152, 0, 0, -160, 0, -80, 0, -16], [0, 89, -29, 0, -273, 0, 183, 0], [-230, 0, 0, -118, 0, -52, 0, -20], [-455, 0, 0, -4, 0, 96, 0, 12], [0, -149, -58, 0, -529, 0, -179, 0], [-198, 0, 0, -146, 0, 48, 0, -30], [0, -43, -60, 0, -317, 0, 17, 0], [-248, 0, 0, -118, 0, 138, 0, 24], [0, 178, -78, 0, -114, 0, 42, 0], [0, -66, -92, 0, -302, 0, 2, 0], [-111, 0, 0, 145, 0, 81, 0, -9], [0, 56, -64, 0, -208, 0, -32, 0], [303, 0, 0, -50, 0, -249, 0, 33], [0, -197, 31, 0, 399, 0, -21, 0], [393, 0, 0, -17, 0, 135, 0, 15], [0, -229, -90, 0, -52, 0, 40, 0], [-214, 0, 0, -174, 0, 92, 0, 58], [0, 43, -12, 0, -9, 0, 201, 0], [-70, 0, 0, -312, 0, -42, 0, 18], [0, 282, -28, 0, -260, 0, -64, 0], [-495, 0, 0, -38, 0, 122, 0, 58], [0, -44, -44, 0, 336, 0, 132, 0], [-91, 0, 0, -102, 0, 139, 0, 53], [0, -111, -80, 0, -294, 0, -186, 0], [-221, 0, 0, 110, 0, -123, 0, 15], [0, 143, -15, 0, -141, 0, 177, 0], [21, 0, 0, 70, 0, -105, 0, 33], [0, 31, -86, 0, 285, 0, 27, 0], [-463, 0, 0, 183, 0, 103, 0, -13], [0, 73, -56, 0, 385, 0, -181, 0], [0, 164, 140, 0, 16, 0, 20, 0], [0, 156, -54, 0, -190, 0, 16, 0], [344, 0, 0, -132, 0, -74, 0, 74], [0, 33, 68, 0, 570, 0, 66, 0], [0, -262, 38, 0, 264, 0, -198, 0], [0, -159, -40, 0, -294, 0, -126, 0], [-559, 0, 0, 168, 0, -89, 0, 5], [-187, 0, 0, 15, 0, -181, 0, 67]]
